"use strict";

const striptags = require("striptags");
const meta = require.main.require("./src/meta");
const user = require.main.require("./src/user");

let library = {};

const renderAdmin = (req, res) => res.render("admin/plugins/oxide");

library.init = ({ router, middleware }, callback) => {
	const app = router;

	app.get("/admin/plugins/oxide", middleware.admin.buildHeader, renderAdmin);
	app.get("/api/admin/plugins/oxide", renderAdmin);

	callback();
};

library.addAdminNavigation = (header, callback) => {
	header.plugins.push({
		route: "/plugins/oxide",
		icon: "fa-paint-brush",
		name: "Oxide Theme",
	});

	callback(null, header);
};

library.getTeasers = (data, callback) => {
	data.teasers.forEach((teaser) =>
		teaser && teaser.content ? (teaser.content = striptags(teaser.content, ["img"])) : ""
	);

	callback(null, data);
};

library.defineWidgetAreas = (areas, callback) => {
	const localAreas = [
		{
			name: "Categories Sidebar",
			template: "categories.tpl",
			location: "sidebar",
		},
		{
			name: "Category Sidebar",
			template: "category.tpl",
			location: "sidebar",
		},
		{
			name: "Topic Sidebar",
			template: "topic.tpl",
			location: "sidebar",
		},
		{
			name: "Categories Header",
			template: "categories.tpl",
			location: "header",
		},
		{
			name: "Category Header",
			template: "category.tpl",
			location: "header",
		},
		{
			name: "Topic Header",
			template: "topic.tpl",
			location: "header",
		},
		{
			name: "Categories Footer",
			template: "categories.tpl",
			location: "footer",
		},
		{
			name: "Category Footer",
			template: "category.tpl",
			location: "footer",
		},
		{
			name: "Topic Footer",
			template: "topic.tpl",
			location: "footer",
		},
	];

	callback(null, [...areas, ...localAreas]);
};

library.getThemeConfig = (config, callback) => {
	meta.settings.get("oxide", (err, settings) => {
		if (err) return callback(err);

		config.hideSubCategories = settings.hideSubCategories === "on";
		config.hideCategoryLastPost = settings.hideCategoryLastPost === "on";
		config.enableQuickReply = settings.enableQuickReply === "on";
		config.enableShowTid = settings.enableShowTid === "on";
		config.enableCategoryIcon = settings.enableCategoryIcon === "on";
		config.enableShowUserAllGroupsInProfile = settings.enableShowUserAllGroupsInProfile === "on";
		config.enableShowIpInUserProfile = settings.enableShowIpInUserProfile === "on";
		config.enableDarkmodeIsDefault = settings.enableDarkmodeIsDefault === "on";
	});

	callback(null, config);
};

library.addUserToTopic = (data, callback) => {
	const { user: reqUser } = data.req;

	if (!reqUser) {
		data.templateData.loggedInUser = {
			uid: 0,
			username: "[[global:guest]]",
			picture: user.getDefaultAvatar(),
			"icon:text": "?",
			"icon:bgColor": "#aaa",
		};

		return callback(null, data);
	}

	user.getUserData(reqUser.uid, (err, userdata) => {
		if (err) return callback(err);

		data.templateData.loggedInUser = userdata;

		callback(null, data);
	});
};

module.exports = library;
